#include "communication/UCommands.hpp"
#include "misc/Utils.hpp"
extern MotorWheel wheelLeft;
extern MotorWheel wheelRight;

void UCommands::setSetpoint (uint8_t* d, U_Component c)
{
	float speed1 = Utils::bytesToFloat(d[COMMAND_POS + 1],
									  d[COMMAND_POS + 2],
									  d[COMMAND_POS + 3],
									  d[COMMAND_POS + 4]);
	switch (c) {
		case ALL_MOTORS: {
			float speed2 = Utils::bytesToFloat(d[COMMAND_POS + 5],
											d[COMMAND_POS + 6],
											d[COMMAND_POS + 7],
											d[COMMAND_POS + 8]);
			wheelLeft.setGearedRPM(speed1, speed1 > 0 ? DIR_BACKOFF : DIR_ADVANCE);
			wheelRight.setGearedRPM(speed2, speed2 > 0 ? DIR_ADVANCE : DIR_BACKOFF);
			
			break;
		}
		case RIGHT_MOTOR: {
			wheelRight.setGearedRPM(speed1, speed1 > 0 ? DIR_ADVANCE : DIR_BACKOFF);
			break;
		}
		case LEFT_MOTOR: {
			wheelLeft.setGearedRPM(speed1, speed1 > 0 ? DIR_BACKOFF : DIR_ADVANCE);
			break;
		}
		default: {
			break;
		}
	}
}

void UCommands::setPIDParameter (uint8_t* d, U_Component c)
{
	float P = Utils::bytesToFloat(d[COMMAND_POS + 1],
									  d[COMMAND_POS + 2],
									  d[COMMAND_POS + 3],
									  d[COMMAND_POS + 4]);
	float I = Utils::bytesToFloat(d[COMMAND_POS + 5],
									  d[COMMAND_POS + 6],
									  d[COMMAND_POS + 7],
									  d[COMMAND_POS + 8]);
	float D = Utils::bytesToFloat(d[COMMAND_POS + 9],
									  d[COMMAND_POS + 10],
									  d[COMMAND_POS + 11],
									  d[COMMAND_POS + 12]);
	int T = Utils::bytesToInt(d[COMMAND_POS + 9],
									  d[COMMAND_POS + 10],
									  d[COMMAND_POS + 11],
									  d[COMMAND_POS + 12]);
	switch (c) {
		case ALL_MOTORS: {
			wheelLeft.PIDSetup(P,I,D,T);
			wheelRight.PIDSetup(P,I,D,T);
			break;
		}
		case RIGHT_MOTOR: {
			wheelLeft.PIDSetup(P,I,D,T);
			break;
		}
		case LEFT_MOTOR: {
			wheelRight.PIDSetup(P,I,D,T);
			break;
		}
		default: {
			break;
		}
	}
}