#ifndef DIFFDRIVE_ARDUINO_ARDUINO_COMMS_H
#define DIFFDRIVE_ARDUINO_ARDUINO_COMMS_H

#include <serial/serial.h>
#include <cstring>
#include "rclcpp/rclcpp.hpp"


typedef enum frametypes {
	VALUE = 0x00, REQUEST = 0x20, RESPONSE = 0x40
} U_FrameType;

typedef enum components {
	DEFAULT_COMPONENT = 0x00, LEFT_MOTOR = 0x10, RIGHT_MOTOR = 0x20, BOTH_MOTORS = 0x30
} U_Component;

typedef enum requests {
	DEFAULT_REQUEST, DRIVE_MOTOR, RESET_TIMESTAMP, DRIVE_ENCODER, PID_PARAMETER
} U_Request;

class ArduinoComms
{


public:

  ArduinoComms()
  : logger_(rclcpp::get_logger("ArduinoCOMS"))
  {  }

  ArduinoComms(const std::string &serial_device, int32_t baud_rate, int32_t timeout_ms)
      : serial_conn_(serial_device, baud_rate, serial::Timeout::simpleTimeout(timeout_ms)), logger_(rclcpp::get_logger("ArduinoCOMS"))
  {  }

  void setup(const std::string &serial_device, int32_t baud_rate, int32_t timeout_ms);
  void sendEmptyMsg();
  void readEncoderValues(double &val_1, double &val_2);
  void setMotorValues(float val_1, float val_2);
  void setPidValues(float k_p, float k_d, float k_i, float k_o);
  const std::vector<uint8_t> buildMsg(uint8_t frame_type, uint8_t frame_component, long time,  std::vector<uint8_t> payload);
  void readSpeed(double &left_speed, double &right_speed);

  bool connected() const { return serial_conn_.isOpen(); }

  void sendMsg(const std::vector<uint8_t> &msg_to_send, bool print_output = false);


private:
  serial::Serial serial_conn_;  ///< Underlying serial connection 
  std::vector<uint8_t> marshalling(float number);
  std::vector<uint8_t> marshalling(uint16_t number);
  std::vector<uint8_t> marshalling(uint32_t number);
  void safeRead(std::vector<uint8_t> &buffer, int bytes_to_read);
  rclcpp::Logger logger_;



};

#endif // DIFFDRIVE_ARDUINO_ARDUINO_COMMS_H